@@ -1,6 +1,8 @@
 package memory
 
-import "time"
+import (
+	"time"
+)
 
 // Config defines the config for storage.
 type Config struct {
