@@ -1,7 +1,7 @@
 package recover //nolint:predeclared // TODO: Rename to some non-builtin
 
 import (
-	"github.com/gofiber/fiber/v2"
+	"github.com/gofiber/fiber/v3"
 )
 
 // Config defines the config for middleware.
@@ -9,17 +9,17 @@ type Config struct {
 	// Next defines a function to skip this middleware when returned true.
 	//
 	// Optional. Default: nil
-	Next func(c *fiber.Ctx) bool
+	Next func(c fiber.Ctx) bool
+
+	// StackTraceHandler defines a function to handle stack trace
+	//
+	// Optional. Default: defaultStackTraceHandler
+	StackTraceHandler func(c fiber.Ctx, e any)
 
 	// EnableStackTrace enables handling stack trace
 	//
 	// Optional. Default: false
 	EnableStackTrace bool
-
-	// StackTraceHandler defines a function to handle stack trace
-	//
-	// Optional. Default: defaultStackTraceHandler
-	StackTraceHandler func(c *fiber.Ctx, e interface{})
 }
 
 // ConfigDefault is the default config
