@@ -1,21 +1,22 @@
 // ⚡️ Fiber is an Express inspired web framework written in Go with ☕️
-// 📄 Github Repository: https://github.com/gofiber/fiber
+// 📄 GitHub Repository: https://github.com/gofiber/fiber
 // 📌 API Documentation: https://docs.gofiber.io
-// ⚠️ This path parser was inspired by ucarion/urlpath (MIT License).
+// ⚠️ This path parser was inspired by https://github.com/ucarion/urlpath
 // 💖 Maintained and modified for Fiber by @renewerner87
 
 package fiber
 
 import (
+	"bytes"
 	"regexp"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
 	"unicode"
 
+	"github.com/gofiber/utils/v2"
 	"github.com/google/uuid"
-
-	"github.com/gofiber/fiber/v2/utils"
 )
 
 // routeParser holds the path segments and param names
@@ -26,23 +27,29 @@ type routeParser struct {
 	plusCount     int             // number of plus parameters, used internally to give the plus parameter its number
 }
 
-// paramsSeg holds the segment metadata
+var routerParserPool = &sync.Pool{
+	New: func() any {
+		return &routeParser{}
+	},
+}
+
+// routeSegment holds the segment metadata
 type routeSegment struct {
 	// const information
-	Const string // constant part of the route
-	// parameter information
-	IsParam     bool   // Truth value that indicates whether it is a parameter or a constant part
-	ParamName   string // name of the parameter for access to it, for wildcards and plus parameters access iterators starting with 1 are added
-	ComparePart string // search part to find the end of the parameter
-	PartCount   int    // how often is the search part contained in the non-param segments? -> necessary for greedy search
-	IsGreedy    bool   // indicates whether the parameter is greedy or not, is used with wildcard and plus
-	IsOptional  bool   // indicates whether the parameter is optional or not
-	// common information
-	IsLast           bool          // shows if the segment is the last one for the route
-	HasOptionalSlash bool          // segment has the possibility of an optional slash
-	Constraints      []*Constraint // Constraint type if segment is a parameter, if not it will be set to noConstraint by default
-	Length           int           // length of the parameter for segment, when its 0 then the length is undetermined
+	Const       string        // constant part of the route
+	ParamName   string        // name of the parameter for access to it, for wildcards and plus parameters access iterators starting with 1 are added
+	ComparePart string        // search part to find the end of the parameter
+	Constraints []*Constraint // Constraint type if segment is a parameter, if not it will be set to noConstraint by default
+	PartCount   int           // how often is the search part contained in the non-param segments? -> necessary for greedy search
+	Length      int           // length of the parameter for segment, when its 0 then the length is undetermined
 	// future TODO: add support for optional groups "/abc(/def)?"
+	// parameter information
+	IsParam    bool // Truth value that indicates whether it is a parameter or a constant part
+	IsGreedy   bool // indicates whether the parameter is greedy or not, is used with wildcard and plus
+	IsOptional bool // indicates whether the parameter is optional or not
+	// common information
+	IsLast           bool // shows if the segment is the last one for the route
+	HasOptionalSlash bool // segment has the possibility of an optional slash
 }
 
 // different special routing signs
@@ -58,20 +65,37 @@ const (
 	paramConstraintSeparator     byte = ';'  // separator of type constraints for a parameter
 	paramConstraintDataStart     byte = '('  // start of data of type constraint for a parameter
 	paramConstraintDataEnd       byte = ')'  // end of data of type constraint for a parameter
-	paramConstraintDataSeparator byte = ','  // separator of datas of type constraint for a parameter
+	paramConstraintDataSeparator byte = ','  // separator of data of type constraint for a parameter
 )
 
 // TypeConstraint parameter constraint types
-type TypeConstraint int16
+type TypeConstraint uint16
 
+// Constraint describes the validation rules that apply to a dynamic route
+// segment when matching incoming requests.
 type Constraint struct {
-	ID            TypeConstraint
-	RegexCompiler *regexp.Regexp
-	Data          []string
+	RegexCompiler     *regexp.Regexp
+	Name              string
+	Data              []string
+	customConstraints []CustomConstraint
+	ID                TypeConstraint
+}
+
+// CustomConstraint is an interface for custom constraints
+type CustomConstraint interface {
+	// Name returns the name of the constraint.
+	// This name is used in the constraint matching.
+	Name() string
+
+	// Execute executes the constraint.
+	// It returns true if the constraint is matched and right.
+	// param is the parameter value to check.
+	// args are the constraint arguments.
+	Execute(param string, args ...string) bool
 }
 
 const (
-	noConstraint TypeConstraint = iota + 1
+	noConstraint TypeConstraint = 1 << iota
 	intConstraint
 	boolConstraint
 	floatConstraint
@@ -88,6 +112,11 @@ const (
 	regexConstraint
 )
 
+const (
+	needOneData = minLenConstraint | maxLenConstraint | lenConstraint | minConstraint | maxConstraint | datetimeConstraint | regexConstraint
+	needTwoData = betweenLenConstraint | rangeConstraint
+)
+
 // list of possible parameter and segment delimiter
 var (
 	// slash has a special role, unlike the other parameters it must not be interpreted as a parameter
@@ -95,26 +124,31 @@ var (
 	// list of greedy parameters
 	greedyParameters = []byte{wildcardParam, plusParam}
 	// list of chars for the parameter recognizing
-	parameterStartChars = []byte{wildcardParam, plusParam, paramStarterChar}
+	parameterStartChars = [256]bool{
+		wildcardParam:    true,
+		plusParam:        true,
+		paramStarterChar: true,
+	}
 	// list of chars of delimiters and the starting parameter name char
 	parameterDelimiterChars = append([]byte{paramStarterChar, escapeChar}, routeDelimiter...)
 	// list of chars to find the end of a parameter
-	parameterEndChars = append([]byte{optionalParam}, parameterDelimiterChars...)
-	// list of parameter constraint start
-	parameterConstraintStartChars = []byte{paramConstraintStart}
-	// list of parameter constraint end
-	parameterConstraintEndChars = []byte{paramConstraintEnd}
-	// list of parameter separator
-	parameterConstraintSeparatorChars = []byte{paramConstraintSeparator}
-	// list of parameter constraint data start
-	parameterConstraintDataStartChars = []byte{paramConstraintDataStart}
-	// list of parameter constraint data end
-	parameterConstraintDataEndChars = []byte{paramConstraintDataEnd}
-	// list of parameter constraint data separator
-	parameterConstraintDataSeparatorChars = []byte{paramConstraintDataSeparator}
+	parameterEndChars = [256]bool{
+		optionalParam:    true,
+		paramStarterChar: true,
+		escapeChar:       true,
+		slashDelimiter:   true,
+		'-':              true,
+		'.':              true,
+	}
 )
 
-// RoutePatternMatch checks if a given path matches a Fiber route pattern.
+// RoutePatternMatch reports whether path matches the provided Fiber route pattern.
+//
+// Patterns use the same syntax as routes registered on an App, including
+// parameters (for example `:id`), wildcards (`*`, `+`), and optional segments.
+// The optional Config argument can be used to control case sensitivity and
+// strict routing behavior. This helper allows checking potential matches
+// without registering a route.
 func RoutePatternMatch(path, pattern string, cfg ...Config) bool {
 	// See logic in (*Route).match and (*App).register
 	var ctxParams [maxParams]string
@@ -137,11 +171,11 @@ func RoutePatternMatch(path, pattern string, cfg ...Config) bool {
 		pattern = "/" + pattern
 	}
 
-	patternPretty := pattern
+	patternPretty := []byte(pattern)
 
 	// Case-sensitive routing, all to lowercase
 	if !config.CaseSensitive {
-		patternPretty = utils.ToLower(patternPretty)
+		patternPretty = utils.ToLowerBytes(patternPretty)
 		path = utils.ToLower(path)
 	}
 	// Strict routing, remove trailing slashes
@@ -149,12 +183,14 @@ func RoutePatternMatch(path, pattern string, cfg ...Config) bool {
 		patternPretty = utils.TrimRight(patternPretty, '/')
 	}
 
-	parser := parseRoute(patternPretty)
+	parser, _ := routerParserPool.Get().(*routeParser) //nolint:errcheck // only contains routeParser
+	parser.reset()
+	patternStr := string(patternPretty)
+	parser.parseRoute(patternStr)
+	defer routerParserPool.Put(parser)
 
-	if patternPretty == "/" && path == "/" {
-		return true
-		// '*' wildcard matches any path
-	} else if patternPretty == "/*" {
+	// '*' wildcard matches any path
+	if (patternStr == "/" && path == "/") || patternStr == "/*" {
 		return true
 	}
 
@@ -165,43 +201,47 @@ func RoutePatternMatch(path, pattern string, cfg ...Config) bool {
 		}
 	}
 	// Check for a simple match
-	patternPretty = RemoveEscapeChar(patternPretty)
-	if len(patternPretty) == len(path) && patternPretty == path {
-		return true
-	}
-	// No match
-	return false
+	patternPretty = RemoveEscapeCharBytes(patternPretty)
+
+	return string(patternPretty) == path
+}
+
+func (parser *routeParser) reset() {
+	parser.segs = parser.segs[:0]
+	parser.params = parser.params[:0]
+	parser.wildCardCount = 0
+	parser.plusCount = 0
 }
 
 // parseRoute analyzes the route and divides it into segments for constant areas and parameters,
 // this information is needed later when assigning the requests to the declared routes
-func parseRoute(pattern string) routeParser {
-	parser := routeParser{}
-
-	part := ""
-	for len(pattern) > 0 {
+func (parser *routeParser) parseRoute(pattern string, customConstraints ...CustomConstraint) {
+	var n int
+	var seg *routeSegment
+	for pattern != "" {
 		nextParamPosition := findNextParamPosition(pattern)
 		// handle the parameter part
 		if nextParamPosition == 0 {
-			processedPart, seg := parser.analyseParameterPart(pattern)
-			parser.params, parser.segs, part = append(parser.params, seg.ParamName), append(parser.segs, seg), processedPart
+			n, seg = parser.analyseParameterPart(pattern, customConstraints...)
+			parser.params, parser.segs = append(parser.params, seg.ParamName), append(parser.segs, seg)
 		} else {
-			processedPart, seg := parser.analyseConstantPart(pattern, nextParamPosition)
-			parser.segs, part = append(parser.segs, seg), processedPart
+			n, seg = parser.analyseConstantPart(pattern, nextParamPosition)
+			parser.segs = append(parser.segs, seg)
 		}
-
-		// reduce the pattern by the processed parts
-		if len(part) == len(pattern) {
-			break
-		}
-		pattern = pattern[len(part):]
+		pattern = pattern[n:]
 	}
 	// mark last segment
 	if len(parser.segs) > 0 {
 		parser.segs[len(parser.segs)-1].IsLast = true
 	}
 	parser.segs = addParameterMetaInfo(parser.segs)
+}
 
+// parseRoute analyzes the route and divides it into segments for constant areas and parameters,
+// this information is needed later when assigning the requests to the declared routes
+func parseRoute(pattern string, customConstraints ...CustomConstraint) routeParser {
+	parser := routeParser{}
+	parser.parseRoute(pattern, customConstraints...)
 	return parser
 }
 
@@ -224,12 +264,12 @@ func addParameterMetaInfo(segs []*routeSegment) []*routeSegment {
 		}
 	}
 
-	// loop from begin to end
-	for i := 0; i < segLen; i++ {
+	// loop from beginning to end
+	for i := range segLen {
 		// check how often the compare part is in the following const parts
 		if segs[i].IsParam {
-			// check if parameter segments are directly after each other and if one of them is greedy
-			// in case the next parameter or the current parameter is not a wildcard it's not greedy, we only want one character
+			// check if parameter segments are directly after each other;
+			// when neither this parameter nor the next parameter are greedy, we only want one character
 			if segLen > i+1 && !segs[i].IsGreedy && segs[i+1].IsParam && !segs[i+1].IsGreedy {
 				segs[i].Length = 1
 			}
@@ -242,7 +282,7 @@ func addParameterMetaInfo(segs []*routeSegment) []*routeSegment {
 					segs[i].PartCount += strings.Count(segs[j].Const, segs[i].ComparePart)
 				}
 			}
-			// check if the end of the segment is a optional slash and then if the segement is optional or the last one
+			// check if the end of the segment is an optional slash and then if the segment is optional or the last one
 		} else if segs[i].Const[len(segs[i].Const)-1] == slashDelimiter && (segs[i].IsLast || (segLen > i+1 && segs[i+1].IsOptional)) {
 			segs[i].HasOptionalSlash = true
 		}
@@ -253,23 +293,28 @@ func addParameterMetaInfo(segs []*routeSegment) []*routeSegment {
 
 // findNextParamPosition search for the next possible parameter start position
 func findNextParamPosition(pattern string) int {
-	nextParamPosition := findNextNonEscapedCharsetPosition(pattern, parameterStartChars)
-	if nextParamPosition != -1 && len(pattern) > nextParamPosition && pattern[nextParamPosition] != wildcardParam {
-		// search for parameter characters for the found parameter start,
-		// if there are more, move the parameter start to the last parameter char
-		for found := findNextNonEscapedCharsetPosition(pattern[nextParamPosition+1:], parameterStartChars); found == 0; {
-			nextParamPosition++
-			if len(pattern) > nextParamPosition {
-				break
-			}
+	// Find the first parameter position
+	next := -1
+	for i := range pattern {
+		if parameterStartChars[pattern[i]] && (i == 0 || pattern[i-1] != escapeChar) {
+			next = i
+			break
 		}
 	}
-
-	return nextParamPosition
+	if next > 0 && pattern[next] != wildcardParam {
+		// checking the found parameterStartChar is a cluster
+		for i := next + 1; i < len(pattern); i++ {
+			if !parameterStartChars[pattern[i]] {
+				return i - 1
+			}
+		}
+		return len(pattern) - 1
+	}
+	return next
 }
 
 // analyseConstantPart find the end of the constant part and create the route segment
-func (*routeParser) analyseConstantPart(pattern string, nextParamPosition int) (string, *routeSegment) {
+func (*routeParser) analyseConstantPart(pattern string, nextParamPosition int) (int, *routeSegment) {
 	// handle the constant part
 	processedPart := pattern
 	if nextParamPosition != -1 {
@@ -277,67 +322,81 @@ func (*routeParser) analyseConstantPart(pattern string, nextParamPosition int) (
 		processedPart = pattern[:nextParamPosition]
 	}
 	constPart := RemoveEscapeChar(processedPart)
-	return processedPart, &routeSegment{
+	return len(processedPart), &routeSegment{
 		Const:  constPart,
 		Length: len(constPart),
 	}
 }
 
 // analyseParameterPart find the parameter end and create the route segment
-func (routeParser *routeParser) analyseParameterPart(pattern string) (string, *routeSegment) {
+func (parser *routeParser) analyseParameterPart(pattern string, customConstraints ...CustomConstraint) (int, *routeSegment) {
 	isWildCard := pattern[0] == wildcardParam
 	isPlusParam := pattern[0] == plusParam
 
-	var parameterEndPosition int
-	if strings.ContainsRune(pattern, rune(paramConstraintStart)) && strings.ContainsRune(pattern, rune(paramConstraintEnd)) {
-		parameterEndPosition = findNextCharsetPositionConstraint(pattern[1:], parameterEndChars)
-	} else {
-		parameterEndPosition = findNextNonEscapedCharsetPosition(pattern[1:], parameterEndChars)
-	}
+	paramEndPosition := 0
+	paramConstraintStartPosition := -1
+	paramConstraintEndPosition := -1
 
-	parameterConstraintStart := -1
-	parameterConstraintEnd := -1
 	// handle wildcard end
-	switch {
-	case isWildCard, isPlusParam:
-		parameterEndPosition = 0
-	case parameterEndPosition == -1:
-		parameterEndPosition = len(pattern) - 1
-	case !isInCharset(pattern[parameterEndPosition+1], parameterDelimiterChars):
-		parameterEndPosition++
-	}
+	if !isWildCard && !isPlusParam {
+		paramEndPosition = -1
+		search := pattern[1:]
+		for i := range search {
+			if paramConstraintStartPosition == -1 && search[i] == paramConstraintStart && (i == 0 || search[i-1] != escapeChar) {
+				paramConstraintStartPosition = i + 1
+				continue
+			}
+			if paramConstraintEndPosition == -1 && search[i] == paramConstraintEnd && (i == 0 || search[i-1] != escapeChar) {
+				paramConstraintEndPosition = i + 1
+				continue
+			}
+			if parameterEndChars[search[i]] {
+				if (paramConstraintStartPosition == -1 && paramConstraintEndPosition == -1) ||
+					(paramConstraintStartPosition != -1 && paramConstraintEndPosition != -1) {
+					paramEndPosition = i
+					break
+				}
+			}
+		}
 
-	// find constraint part if exists in the parameter part and remove it
-	if parameterEndPosition > 0 {
-		parameterConstraintStart = findNextNonEscapedCharsetPosition(pattern[0:parameterEndPosition], parameterConstraintStartChars)
-		parameterConstraintEnd = findLastCharsetPosition(pattern[0:parameterEndPosition+1], parameterConstraintEndChars)
+		switch {
+		case paramEndPosition == -1:
+			paramEndPosition = len(pattern) - 1
+		case bytes.IndexByte(parameterDelimiterChars, pattern[paramEndPosition+1]) == -1:
+			paramEndPosition++
+		default:
+			// do nothing
+		}
 	}
 
 	// cut params part
-	processedPart := pattern[0 : parameterEndPosition+1]
+	processedPart := pattern[0 : paramEndPosition+1]
+	n := paramEndPosition + 1
 	paramName := RemoveEscapeChar(GetTrimmedParam(processedPart))
 
 	// Check has constraint
 	var constraints []*Constraint
 
-	if hasConstraint := parameterConstraintStart != -1 && parameterConstraintEnd != -1; hasConstraint {
-		constraintString := pattern[parameterConstraintStart+1 : parameterConstraintEnd]
-		userConstraints := splitNonEscaped(constraintString, string(parameterConstraintSeparatorChars))
+	if hasConstraint := paramConstraintStartPosition != -1 && paramConstraintEndPosition != -1; hasConstraint {
+		constraintString := pattern[paramConstraintStartPosition+1 : paramConstraintEndPosition]
+		userConstraints := splitNonEscaped(constraintString, paramConstraintSeparator)
 		constraints = make([]*Constraint, 0, len(userConstraints))
 
 		for _, c := range userConstraints {
-			start := findNextNonEscapedCharsetPosition(c, parameterConstraintDataStartChars)
-			end := findLastCharsetPosition(c, parameterConstraintDataEndChars)
+			start := findNextNonEscapedCharPosition(c, paramConstraintDataStart)
+			end := strings.LastIndexByte(c, paramConstraintDataEnd)
 
 			// Assign constraint
 			if start != -1 && end != -1 {
 				constraint := &Constraint{
-					ID: getParamConstraintType(c[:start]),
+					ID:                getParamConstraintType(c[:start]),
+					Name:              c[:start],
+					customConstraints: customConstraints,
 				}
 
 				// remove escapes from data
 				if constraint.ID != regexConstraint {
-					constraint.Data = splitNonEscaped(c[start+1:end], string(parameterConstraintDataSeparatorChars))
+					constraint.Data = splitNonEscaped(c[start+1:end], paramConstraintDataSeparator)
 					if len(constraint.Data) == 1 {
 						constraint.Data[0] = RemoveEscapeChar(constraint.Data[0])
 					} else if len(constraint.Data) == 2 { // This is fine, we simply expect two parts
@@ -355,28 +414,30 @@ func (routeParser *routeParser) analyseParameterPart(pattern string) (string, *r
 				constraints = append(constraints, constraint)
 			} else {
 				constraints = append(constraints, &Constraint{
-					ID:   getParamConstraintType(c),
-					Data: []string{},
+					ID:                getParamConstraintType(c),
+					Data:              []string{},
+					Name:              c,
+					customConstraints: customConstraints,
 				})
 			}
 		}
 
-		paramName = RemoveEscapeChar(GetTrimmedParam(pattern[0:parameterConstraintStart]))
+		paramName = RemoveEscapeChar(GetTrimmedParam(pattern[0:paramConstraintStartPosition]))
 	}
 
 	// add access iterator to wildcard and plus
 	if isWildCard {
-		routeParser.wildCardCount++
-		paramName += strconv.Itoa(routeParser.wildCardCount)
+		parser.wildCardCount++
+		paramName += strconv.Itoa(parser.wildCardCount)
 	} else if isPlusParam {
-		routeParser.plusCount++
-		paramName += strconv.Itoa(routeParser.plusCount)
+		parser.plusCount++
+		paramName += strconv.Itoa(parser.plusCount)
 	}
 
 	segment := &routeSegment{
 		ParamName:  paramName,
 		IsParam:    true,
-		IsOptional: isWildCard || pattern[parameterEndPosition] == optionalParam,
+		IsOptional: isWildCard || pattern[paramEndPosition] == optionalParam,
 		IsGreedy:   isWildCard || isPlusParam,
 	}
 
@@ -384,101 +445,59 @@ func (routeParser *routeParser) analyseParameterPart(pattern string) (string, *r
 		segment.Constraints = constraints
 	}
 
-	return processedPart, segment
+	return n, segment
 }
 
-// isInCharset check is the given character in the charset list
-func isInCharset(searchChar byte, charset []byte) bool {
-	for _, char := range charset {
-		if char == searchChar {
-			return true
+// findNextNonEscapedCharPosition searches the next char position and skips the escaped characters
+func findNextNonEscapedCharPosition(search string, char byte) int {
+	for i := 0; i < len(search); i++ {
+		if search[i] == char && (i == 0 || search[i-1] != escapeChar) {
+			return i
 		}
 	}
-	return false
-}
-
-// findNextCharsetPosition search the next char position from the charset
-func findNextCharsetPosition(search string, charset []byte) int {
-	nextPosition := -1
-	for _, char := range charset {
-		if pos := strings.IndexByte(search, char); pos != -1 && (pos < nextPosition || nextPosition == -1) {
-			nextPosition = pos
-		}
-	}
-
-	return nextPosition
-}
-
-// findNextCharsetPosition search the last char position from the charset
-func findLastCharsetPosition(search string, charset []byte) int {
-	lastPosition := -1
-	for _, char := range charset {
-		if pos := strings.LastIndexByte(search, char); pos != -1 && (pos < lastPosition || lastPosition == -1) {
-			lastPosition = pos
-		}
-	}
-
-	return lastPosition
-}
-
-// findNextCharsetPositionConstraint search the next char position from the charset
-// unlike findNextCharsetPosition, it takes care of constraint start-end chars to parse route pattern
-func findNextCharsetPositionConstraint(search string, charset []byte) int {
-	constraintStart := findNextNonEscapedCharsetPosition(search, parameterConstraintStartChars)
-	constraintEnd := findNextNonEscapedCharsetPosition(search, parameterConstraintEndChars)
-	nextPosition := -1
-
-	for _, char := range charset {
-		pos := strings.IndexByte(search, char)
-
-		if pos != -1 && (pos < nextPosition || nextPosition == -1) {
-			if (pos > constraintStart && pos > constraintEnd) || (pos < constraintStart && pos < constraintEnd) {
-				nextPosition = pos
-			}
-		}
-	}
-
-	return nextPosition
-}
-
-// findNextNonEscapedCharsetPosition search the next char position from the charset and skip the escaped characters
-func findNextNonEscapedCharsetPosition(search string, charset []byte) int {
-	pos := findNextCharsetPosition(search, charset)
-	for pos > 0 && search[pos-1] == escapeChar {
-		if len(search) == pos+1 {
-			// escaped character is at the end
-			return -1
-		}
-		nextPossiblePos := findNextCharsetPosition(search[pos+1:], charset)
-		if nextPossiblePos == -1 {
-			return -1
-		}
-		// the previous character is taken into consideration
-		pos = nextPossiblePos + pos + 1
-	}
-
-	return pos
+	return -1
 }
 
 // splitNonEscaped slices s into all substrings separated by sep and returns a slice of the substrings between those separators
 // This function also takes a care of escape char when splitting.
-func splitNonEscaped(s, sep string) []string {
+func splitNonEscaped(s string, sep byte) []string {
 	var result []string
-	i := findNextNonEscapedCharsetPosition(s, []byte(sep))
+	i := findNextNonEscapedCharPosition(s, sep)
 
 	for i > -1 {
 		result = append(result, s[:i])
-		s = s[i+len(sep):]
-		i = findNextNonEscapedCharsetPosition(s, []byte(sep))
+		s = s[i+1:]
+		i = findNextNonEscapedCharPosition(s, sep)
 	}
 
 	return append(result, s)
 }
 
+func hasPartialMatchBoundary(path string, matchedLength int) bool {
+	if matchedLength < 0 || matchedLength > len(path) {
+		return false
+	}
+	if matchedLength == len(path) {
+		return true
+	}
+	if matchedLength == 0 {
+		return false
+	}
+	if path[matchedLength-1] == slashDelimiter {
+		return true
+	}
+	if matchedLength < len(path) && path[matchedLength] == slashDelimiter {
+		return true
+	}
+
+	return false
+}
+
 // getMatch parses the passed url and tries to match it against the route segments and determine the parameter positions
-func (routeParser *routeParser) getMatch(detectionPath, path string, params *[maxParams]string, partialCheck bool) bool { //nolint: revive // Accepting a bool param is fine here
+func (parser *routeParser) getMatch(detectionPath, path string, params *[maxParams]string, partialCheck bool) bool { //nolint:revive // Accepting a bool param is fine here
+	originalDetectionPath := detectionPath
 	var i, paramsIterator, partLen int
-	for _, segment := range routeParser.segs {
+	for _, segment := range parser.segs {
 		partLen = len(detectionPath)
 		// check const segment
 		if !segment.IsParam {
@@ -487,7 +506,7 @@ func (routeParser *routeParser) getMatch(detectionPath, path string, params *[ma
 			// check if the end of the segment is an optional slash
 			if segment.HasOptionalSlash && partLen == i-1 && detectionPath == segment.Const[:i-1] {
 				i--
-			} else if !(i <= partLen && detectionPath[:i] == segment.Const) {
+			} else if i > partLen || detectionPath[:i] != segment.Const {
 				return false
 			}
 		} else {
@@ -499,7 +518,7 @@ func (routeParser *routeParser) getMatch(detectionPath, path string, params *[ma
 			// take over the params positions
 			params[paramsIterator] = path[:i]
 
-			if !(segment.IsOptional && i == 0) {
+			if !segment.IsOptional || i != 0 {
 				// check constraint
 				for _, c := range segment.Constraints {
 					if matched := c.CheckConstraint(params[paramsIterator]); !matched {
@@ -516,8 +535,14 @@ func (routeParser *routeParser) getMatch(detectionPath, path string, params *[ma
 			detectionPath, path = detectionPath[i:], path[i:]
 		}
 	}
-	if detectionPath != "" && !partialCheck {
-		return false
+	if detectionPath != "" {
+		if !partialCheck {
+			return false
+		}
+		consumedLength := len(originalDetectionPath) - len(detectionPath)
+		if !hasPartialMatchBoundary(originalDetectionPath, consumedLength) {
+			return false
+		}
 	}
 
 	return true
@@ -573,11 +598,12 @@ func findGreedyParamLen(s string, searchCount int, segment *routeSegment) int {
 	// check all from right to left segments
 	for i := segment.PartCount; i > 0 && searchCount > 0; i-- {
 		searchCount--
-		if constPosition := strings.LastIndex(s, segment.ComparePart); constPosition != -1 {
-			s = s[:constPosition]
-		} else {
+
+		constPosition := strings.LastIndex(s, segment.ComparePart)
+		if constPosition == -1 {
 			break
 		}
+		s = s[:constPosition]
 	}
 
 	return len(s)
@@ -599,12 +625,36 @@ func GetTrimmedParam(param string) string {
 	return param[start:end]
 }
 
-// RemoveEscapeChar remove escape characters
+// RemoveEscapeChar removes escape characters
 func RemoveEscapeChar(word string) string {
-	if strings.IndexByte(word, escapeChar) != -1 {
-		return strings.ReplaceAll(word, string(escapeChar), "")
+	// Fast path: check if there are any escape characters first
+	escapeIdx := strings.IndexByte(word, '\\')
+	if escapeIdx == -1 {
+		return word // No escape chars, return original string without allocation
 	}
-	return word
+
+	// Slow path: copy and remove escape characters
+	b := []byte(word)
+	dst := escapeIdx
+	for src := escapeIdx + 1; src < len(b); src++ {
+		if b[src] != '\\' {
+			b[dst] = b[src]
+			dst++
+		}
+	}
+	return string(b[:dst])
+}
+
+// RemoveEscapeCharBytes removes escape characters
+func RemoveEscapeCharBytes(word []byte) []byte {
+	dst := 0
+	for src := range word {
+		if word[src] != '\\' {
+			word[dst] = word[src]
+			dst++
+		}
+	}
+	return word[:dst]
 }
 
 func getParamConstraintType(constraintPart string) TypeConstraint {
@@ -617,7 +667,7 @@ func getParamConstraintType(constraintPart string) TypeConstraint {
 		return floatConstraint
 	case ConstraintAlpha:
 		return alphaConstraint
-	case ConstraintGuid:
+	case ConstraintGUID:
 		return guidConstraint
 	case ConstraintMinLen, ConstraintMinLenLower:
 		return minLenConstraint
@@ -642,31 +692,36 @@ func getParamConstraintType(constraintPart string) TypeConstraint {
 	}
 }
 
-//nolint:errcheck // TODO: Properly check _all_ errors in here, log them & immediately return
+// CheckConstraint validates if a param matches the given constraint
+// Returns true if the param passes the constraint check, false otherwise
+//
+//nolint:errcheck // TODO: Properly check _all_ errors in here, log them or immediately return
 func (c *Constraint) CheckConstraint(param string) bool {
-	var err error
-	var num int
-
-	// check data exists
-	needOneData := []TypeConstraint{minLenConstraint, maxLenConstraint, lenConstraint, minConstraint, maxConstraint, datetimeConstraint, regexConstraint}
-	needTwoData := []TypeConstraint{betweenLenConstraint, rangeConstraint}
-
-	for _, data := range needOneData {
-		if c.ID == data && len(c.Data) == 0 {
-			return false
+	// First check if there's a custom constraint with the same name
+	// This allows custom constraints to override built-in constraints
+	for _, cc := range c.customConstraints {
+		if cc.Name() == c.Name {
+			return cc.Execute(param, c.Data...)
 		}
 	}
 
-	for _, data := range needTwoData {
-		if c.ID == data && len(c.Data) < 2 {
-			return false
-		}
+	var (
+		err error
+		num int
+	)
+
+	// Validate constraint has required data
+	if c.ID&needOneData != 0 && len(c.Data) == 0 {
+		return false
+	}
+
+	if c.ID&needTwoData != 0 && len(c.Data) < 2 {
+		return false
 	}
 
-	// check constraints
 	switch c.ID {
 	case noConstraint:
-		// Nothing to check
+		return true
 	case intConstraint:
 		_, err = strconv.Atoi(param)
 	case boolConstraint:
@@ -710,14 +765,14 @@ func (c *Constraint) CheckConstraint(param string) bool {
 		data, _ := strconv.Atoi(c.Data[0])
 		num, err = strconv.Atoi(param)
 
-		if num < data {
+		if err != nil || num < data {
 			return false
 		}
 	case maxConstraint:
 		data, _ := strconv.Atoi(c.Data[0])
 		num, err = strconv.Atoi(param)
 
-		if num > data {
+		if err != nil || num > data {
 			return false
 		}
 	case rangeConstraint:
@@ -725,15 +780,23 @@ func (c *Constraint) CheckConstraint(param string) bool {
 		data2, _ := strconv.Atoi(c.Data[1])
 		num, err = strconv.Atoi(param)
 
-		if num < data || num > data2 {
+		if err != nil || num < data || num > data2 {
 			return false
 		}
 	case datetimeConstraint:
 		_, err = time.Parse(c.Data[0], param)
+		if err != nil {
+			return false
+		}
 	case regexConstraint:
+		if c.RegexCompiler == nil {
+			return false
+		}
 		if match := c.RegexCompiler.MatchString(param); !match {
 			return false
 		}
+	default:
+		return false
 	}
 
 	return err == nil
