@@ -1,5 +1,5 @@
 // ⚡️ Fiber is an Express inspired web framework written in Go with ☕️
-// 📝 Github Repository: https://github.com/gofiber/fiber
+// 📝 GitHub Repository: https://github.com/gofiber/fiber
 // 📌 API Documentation: https://docs.gofiber.io
 
 package fiber
@@ -10,8 +10,8 @@ import (
 
 type routeTestCase struct {
 	url          string
-	match        bool
 	params       []string
+	match        bool
 	partialCheck bool
 }
 
@@ -248,7 +248,7 @@ func init() {
 					{url: "/fooba", params: nil, match: false},
 				},
 			},
-			// chars in front of an parameter
+			// chars in front of a parameter
 			{
 				pattern: "/name::name",
 				testCases: []routeTestCase{
@@ -335,13 +335,39 @@ func init() {
 					{url: "/api_ignore/v1/entity", params: nil, match: false},
 				},
 			},
+			{
+				pattern: "/partialCheck/foo/",
+				testCases: []routeTestCase{
+					{url: "/partialCheck/foo/", params: nil, match: true, partialCheck: true},
+					{url: "/partialCheck/foo/bar", params: nil, match: true, partialCheck: true},
+					{url: "/partialCheck/foo/bar/baz", params: nil, match: true, partialCheck: true},
+					{url: "/partialCheck/foobar", params: nil, match: false, partialCheck: true},
+				},
+			},
 			{
 				pattern: "/partialCheck/foo/bar/:param",
 				testCases: []routeTestCase{
 					{url: "/partialCheck/foo/bar/test", params: []string{"test"}, match: true, partialCheck: true},
 					{url: "/partialCheck/foo/bar/test/test2", params: []string{"test"}, match: true, partialCheck: true},
 					{url: "/partialCheck/foo/bar", params: nil, match: false, partialCheck: true},
-					{url: "/partiaFoo", params: nil, match: false, partialCheck: true},
+					{url: "/partialFoo", params: nil, match: false, partialCheck: true},
+				},
+			},
+			{
+				pattern: "/partialCheck/foo",
+				testCases: []routeTestCase{
+					{url: "/partialCheck/foo", params: nil, match: true, partialCheck: true},
+					{url: "/partialCheck/foo/", params: nil, match: true, partialCheck: true},
+					{url: "/partialCheck/foo/bar", params: nil, match: true, partialCheck: true},
+					{url: "/partialCheck/foobar", params: nil, match: false, partialCheck: true},
+				},
+			},
+			{
+				pattern: "/partialCheck/:param",
+				testCases: []routeTestCase{
+					{url: "/partialCheck/value", params: []string{"value"}, match: true, partialCheck: true},
+					{url: "/partialCheck/value/", params: []string{"value"}, match: true, partialCheck: true},
+					{url: "/partialCheck/value/next", params: []string{"value"}, match: true, partialCheck: true},
 				},
 			},
 			{
@@ -713,6 +739,14 @@ func init() {
 					{url: "/api/v1/", params: []string{""}, match: true},
 				},
 			},
+			// Add test case for RegexCompiler == nil
+			{
+				pattern: "/api/v1/:param<regex(\\d+)>",
+				testCases: []routeTestCase{
+					{url: "/api/v1/123", params: []string{"123"}, match: true},
+					{url: "/api/v1/abc", params: nil, match: false},
+				},
+			},
 		}...,
 	)
 }
